using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text.RegularExpressions;

namespace ChemSpiderMassSpecAPI
{
	public partial class _Default : System.Web.UI.Page
	{
		private chemspider.MassSpecAPI massSpec = new chemspider.MassSpecAPI();
	
		protected void Page_Load(object sender, EventArgs e)
		{
			if(!Page.IsPostBack)
			{
				string[] availableDatabases = massSpec.GetDatabases();
				searchDatabases.DataSource = availableDatabases;
				DataBind();
			}
		}

		private string[] GetSelectedDatabases()
		{
			List<string> selectedDatabases = new List<string>();
			foreach (ListItem item in searchDatabases.Items)
			{
				if (item.Selected)
				{
					selectedDatabases.Add(item.Value);
				}
			}
			
			return selectedDatabases.ToArray();
		}

		protected void searchByFormula2Button_Click(object sender, EventArgs e)
		{
			string formula = searchByFormula2Text.Text;
			if(!string.IsNullOrEmpty(formula))
			{
				string[] compounds = massSpec.SearchByFormula2(formula);
				ShowCompounds(compounds);
			}
			else
			{
				ShowErrorMessage("Nothing to search!");
			}
		}

		protected void searchByFormulaButton_Click(object sender, EventArgs e)
		{
			string formula = searchByFormulaText.Text;
			if (!string.IsNullOrEmpty(formula))
			{
				string[] selectedDatabases = GetSelectedDatabases();
				string[] compounds = massSpec.SearchByFormula(formula, selectedDatabases);
				ShowCompounds(compounds);
			}
			else
			{
				ShowErrorMessage("Nothing to search!");
			}
		}

		protected void searchByMass2Button_Click(object sender, EventArgs e)
		{
			string mass = searchByMass2Text.Text;
			if (!string.IsNullOrEmpty(mass))
			{
				string range = searchByRange2Text.Text;
				if (!string.IsNullOrEmpty(range))
				{
					double d_mass;
					double d_range;
					if( Double.TryParse(mass, out d_mass) && Double.TryParse(range, out d_range) )
					{
						string[] compounds = massSpec.SearchByMass2(d_mass, d_range);
						ShowCompounds(compounds);
					}
					else
					{
						ShowErrorMessage("Incorrect input parameters!");
					}
				}
				else
				{
					ShowErrorMessage("Specify a range!");
				}
			}
			else
			{
				ShowErrorMessage("Nothing to search!");
			}
		}

		protected void searchByMassButton_Click(object sender, EventArgs e)
		{
			string mass = searchByMassText.Text;
			if (!string.IsNullOrEmpty(mass))
			{
				string range = searchByRangeText.Text;
				if (!string.IsNullOrEmpty(range))
				{
					double d_mass;
					double d_range;
					if (Double.TryParse(mass, out d_mass) && Double.TryParse(range, out d_range))
					{
						string[] selectedDatabases = GetSelectedDatabases();
						string[] compounds = massSpec.SearchByMass(d_mass, d_range, selectedDatabases);
						ShowCompounds(compounds);
					}
					else
					{
						ShowErrorMessage("Incorrect input parameters!");
					}
				}
				else
				{
					ShowErrorMessage("Specify a range!");
				}
			}
			else
			{
				ShowErrorMessage("Nothing to search!");
			}
		}

		private void ShowCompounds(string[] compounds)
		{
			if(compounds != null && compounds.Length > 0 )
			{
				searchResult.Text = string.Format("{0} hits found.", compounds.Length);
				foreach(string id in compounds)
				{
					string[] details = massSpec.GetCompoundDetails(id);

					HtmlTableRow row = new HtmlTableRow();
					resultTable.Rows.Add(row);

					HtmlTableCell cell = new HtmlTableCell();
					HyperLink link = new HyperLink();
					link.NavigateUrl = string.Format("http://www.chemspider.com/Chemical-Structure.{0}.html", id);
					link.Target = "_blank";
					link.Text = id;
					cell.Controls.Add(link);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(decodeACDSymbols(details[0]));
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[1]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[2]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[3]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[4]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[5]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[6]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[7]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[8]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[9]);
					row.Cells.Add(cell);

					cell = new HtmlTableCell();
					cell.InnerHtml = Convert(details[10]);
					row.Cells.Add(cell);
				}

				resultTable.Visible = true;
			}
			else
			{
				searchResult.Text = "No hits found.";
			}

			searchResult.Visible = true;
		}

		private void ShowErrorMessage(string message)
		{
			errorMessage.Text = message;
			errorMessage.Visible = true;
		}

		private string Convert(string date)
		{
			if(!string.IsNullOrEmpty(date))
				return date;

			return "&nbsp;";
		}

		static public string decodeACDSymbols(string encoded)
		{
			// Superscript
			Regex rx = new Regex(@"\^{(.*?)}");

			Match m = rx.Match(encoded);
			while (m.Success)
			{
				string repl = m.Groups[1].Captures[0].Value;
				repl = "<sup>" + repl + "</sup>";
				encoded = rx.Replace(encoded, repl, 1);
				m = rx.Match(encoded);
			}

			// Subscript
			Regex rx2 = new Regex(@"_{(.*?)}");
			Match m2 = rx2.Match(encoded);
			while (m2.Success)
			{
				string repl = m2.Groups[1].Captures[0].Value;
				repl = "<sub>" + repl + "</sub>";
				encoded = rx2.Replace(encoded, repl, 1);
				m2 = rx2.Match(encoded);
			}

			return encoded;
		}
	}
}
